;; Generated package description from company-ebdb.el  -*- no-byte-compile: t -*-
(define-package "company-ebdb" "1.1" "company-mode completion backend for EBDB in message-mode" '((company "0.9.4") (ebdb "0.2")) :commit "f4b8ad408a068f2c3d07782cb111db4d62fd69d8" :url "https://elpa.gnu.org/packages/company-ebdb.html" :authors '(("Jan Tatarik" . "jan.tatarik@gmail.com")) :maintainer '("Eric Abrahamsen" . "eric@ericabrahamsen.net"))
